==> oer-0.0.1-20190731.023331-5-sources.jar: com/xdja/pki/itsca/oer/app/data/SignedExternalPayloadBuilder.java <==
package com.xdja.pki.itsca.oer.app.data;

import com.xdja.pki.itsca.oer.app.SignatureBuild;
import com.xdja.pki.itsca.oer.asn1.*;
import com.xdja.pki.itsca.oer.asn1.base.Int;
import com.xdja.pki.itsca.oer.utils.BCUtils;
import com.xdja.pki.itsca.oer.utils.TimeUtils;

import java.io.IOException;
import java.security.PrivateKey;

/***
 *--- 4. SignedExternalPayload
 * SignedExternalPayload ::= SecuredMessage
 * SecuredMessage ::= SEQUENCE{
 *     version  Uint8,
 *     payload  Payload
 * }
 * Payload ::= CHOICE{
 *     unSecuredData   Opaque,
 *     signedData      SignedData,  --- 签名消息体
 *     encData         EncryptedData
 * }
 * SignedData ::= SEQUENCE{
 *     signer  SignerInfo,  ---签名者为 “OBU 设备注册证书”
 *     tbs     TBSData,    ---授权证书申请 hash
 *     sign    Signature   ---使用 “OBU 设备注册证书” 对应私钥进行签名 原文为 “注册证书 hash + tbsData hash”
 * }
 * TBSData ::= SEQUENCE{
 *     headerInfo  HeaderInfo,
 *     data        OCTET STRING(SIZE(0..MAX))  Optional,
 *     extHash     OCTET STRING(SIZE(32))  Optional,  ---为 SharedATRequest 的hash值
 * }
 */
public class SignedExternalPayloadBuilder {
    /**
     * @param sharedATRequest
     * @param ecCert          注册证书
     * @param ecPrivateKey    注册证书签名私钥
     */
    public static SecuredMessage builder(SharedATRequest sharedATRequest, Certificate ecCert, PrivateKey ecPrivateKey) throws IOException {
        SignedData signedData = new SignedData();
        //拼凑SignInfo
        SequenceOfCertificate sequenceOfCertificate = new SequenceOfCertificate();
        sequenceOfCertificate.addCertificate(ecCert);
        SignerInfo signerInfo = new SignerInfo(sequenceOfCertificate);
        signedData.setSignerInfo(signerInfo);
        //TBSData
        TBSData tbsData = new TBSData();
        HeaderInfo headerInfo = new HeaderInfo();
        headerInfo.setItsAid(new Int(40965));
        headerInfo.setGenTime(new Time64(TimeUtils.getNowTime()));
        tbsData.setHeaderInfo(headerInfo);
        byte[] bytes = BCUtils.sm3DigestByBC(sharedATRequest.getEncode());
        tbsData.setExtHash(bytes);
        signedData.setTbs(tbsData);
        //Signature
        Signature signature = SignatureBuild.build(ecPrivateKey, tbsData);
        signedData.setSign(signature);
        SecuredMessage securedMessage = new SecuredMessage();
        Payload payload = new Payload(signedData);
        securedMessage.setPayload(payload);
        return securedMessage;
    }
}

==> oer-0.0.2-20190820.092159-13-sources.jar: com/xdja/pki/itsca/oer/app/data/SignedExternalPayloadBuilder.java <==
package com.xdja.pki.itsca.oer.app.data;

import com.xdja.pki.gmssl.crypto.utils.GMSSLSM3DigestUtils;
import com.xdja.pki.itsca.oer.app.SignatureBuild;
import com.xdja.pki.itsca.oer.asn1.*;
import com.xdja.pki.itsca.oer.asn1.base.Int;
import com.xdja.pki.itsca.oer.utils.BCUtils;
import com.xdja.pki.itsca.oer.utils.TimeUtils;

import java.io.IOException;
import java.security.PrivateKey;

/***
 *--- 4. SignedExternalPayload
 * SignedExternalPayload ::= SecuredMessage
 * SecuredMessage ::= SEQUENCE{
 *     version  Uint8,
 *     payload  Payload
 * }
 * Payload ::= CHOICE{
 *     unSecuredData   Opaque,
 *     signedData      SignedData,  --- 签名消息体
 *     encData         EncryptedData
 * }
 * SignedData ::= SEQUENCE{
 *     signer  SignerInfo,  ---签名者为 “OBU 设备注册证书”
 *     tbs     TBSData,    ---授权证书申请 hash
 *     sign    Signature   ---使用 “OBU 设备注册证书” 对应私钥进行签名 原文为 “注册证书 hash + tbsData hash”
 * }
 * TBSData ::= SEQUENCE{
 *     headerInfo  HeaderInfo,
 *     data        OCTET STRING(SIZE(0..MAX))  Optional,
 *     extHash     OCTET STRING(SIZE(32))  Optional,  ---为 SharedATRequest 的hash值
 * }
 */
public class SignedExternalPayloadBuilder {
    /**
     * @param sharedATRequest
     * @param ecCert          注册证书
     * @param ecPrivateKey    注册证书签名私钥
     */
    public static SecuredMessage builder(SharedATRequest sharedATRequest, Certificate ecCert, PrivateKey ecPrivateKey) throws Exception {
        SignedData signedData = new SignedData();
        //拼凑SignInfo
        SequenceOfCertificate sequenceOfCertificate = new SequenceOfCertificate();
        sequenceOfCertificate.addCertificate(ecCert);
        SignerInfo signerInfo = new SignerInfo(sequenceOfCertificate);
        signedData.setSignerInfo(signerInfo);
        //TBSData
        TBSData tbsData = new TBSData();
        HeaderInfo headerInfo = new HeaderInfo();
        headerInfo.setItsAid(new Int(40965));
        headerInfo.setGenTime(new Time64(TimeUtils.getNowTime()));
        tbsData.setHeaderInfo(headerInfo);
        //byte[] bytes = BCUtils.sm3DigestByBC(sharedATRequest.getEncode());
        byte[] bytes = GMSSLSM3DigestUtils.digestByBC(sharedATRequest.getEncode());
        tbsData.setExtHash(bytes);
        signedData.setTbs(tbsData);
        //Signature
        Signature signature = SignatureBuild.build(ecPrivateKey, tbsData);
        signedData.setSign(signature);
        SecuredMessage securedMessage = new SecuredMessage();
        Payload payload = new Payload(signedData);
        securedMessage.setPayload(payload);
        return securedMessage;
    }
}
